@@ -1043,6 +1043,16 @@ class Message(Protocol):
         """
         return self.getTagAttr('origin-id', namespace=NS_SID, attr='id')
 
+    def getStanzaIDAttrs(self):
+        """
+        Return the stanza-id attributes of the message
+        """
+        try:
+            attrs = self.getTag('stanza-id', namespace=NS_SID).getAttrs()
+        except Exception:
+            return None, None
+        return attrs['id'], attrs['by']
+
     def setBody(self, val):
         """
         Set the text of the message"""
@@ -1390,6 +1400,10 @@ class Hashes(Node):
                 hash_ = hl.hexdigest()
         return hash_
 
+    def addHash(self, hash_, algo):
+        self.setAttr('algo', algo)
+        self.setData(hash_)
+
 class Hashes2(Node):
     """
     Hash elements for various XEPs as defined in XEP-300
